@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Free Mandatory Access Control.
  *
- * Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.5.2   2007/12/05
+ * Version: 1.5.3   2008/01/31
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -58,7 +58,7 @@ static int AddNoUmountACL(const char *dir, const int is_delete)
 	}
 	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->dir = saved_dir;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = no_umount_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -100,7 +100,6 @@ int SAKURA_MayUmount(struct vfsmount *mnt)
 	if (!is_enforce) error = 0;
 	return error;
 }
-EXPORT_SYMBOL(SAKURA_MayUmount);
 
 int AddNoUmountPolicy(char *data, const int is_delete)
 {
